@@ -49,33 +49,29 @@ let options =
 let f { files; output_file; include_dirs } =
   let code =
     {|
-//Provides: caml_create_file_extern
-function caml_create_file_extern(name,content){
-  if(joo_global_object.caml_create_file)
-    joo_global_object.caml_create_file(name,content);
+//Provides: jsoo_create_file_extern
+function jsoo_create_file_extern(name,content){
+  if(globalThis.jsoo_create_file)
+    globalThis.jsoo_create_file(name,content);
   else {
-    if(!joo_global_object.caml_fs_tmp) joo_global_object.caml_fs_tmp = [];
-    joo_global_object.caml_fs_tmp.push({name:name,content:content});
+    if(!globalThis.caml_fs_tmp) globalThis.caml_fs_tmp = [];
+    globalThis.caml_fs_tmp.push({name:name,content:content});
   }
   return 0;
 }
 |}
   in
   let fragments = Linker.parse_string code in
-  List.iter fragments ~f:(fun fr -> Linker.load_fragment ~filename:"<dummy>" fr);
+  Linker.load_fragments ~target_env:Isomorphic ~filename:"<dummy>" fragments;
   let instr =
-    Pseudo_fs.f
-      ~prim:`caml_create_file_extern
-      ~cmis:StringSet.empty
-      ~files
-      ~paths:include_dirs
+    Pseudo_fs.f ~prim:`create_file_extern ~cmis:StringSet.empty ~files ~paths:include_dirs
   in
   let code = Code.prepend Code.empty instr in
   Filename.gen_file output_file (fun chan ->
       let pfs_fmt = Pretty_print.to_out_channel chan in
       Driver.f
         ~standalone:true
-        ~global:`Auto
+        ~wrap_with_fun:`Iife
         pfs_fmt
         (Parse_bytecode.Debug.create ~toplevel:false false)
         code)
