@@ -24,7 +24,7 @@
     binary data in an ASCII string format by translating it into a radix-64
     representation. It is specified in RFC 4648.
 
-    {e Release 3.11.0 - https://ocsigen.github.io/js_of_ocaml} *)
+    {e Release 4.0.0 - https://ocsigen.org/js_of_ocaml/latest/manual/overview} *)
 
 type ('a, 'b) result =
   | Ok of 'a
