@@ -259,7 +259,7 @@ type array_or_not =
 
 type constant =
   | String of string
-  | IString of string
+  | NativeString of string
   | Float of float
   | Float_array of float array
   | Int64 of int64
@@ -269,7 +269,7 @@ type constant =
 let rec constant_equal a b =
   match a, b with
   | String a, String b -> Some (String.equal a b)
-  | IString a, IString b -> Some (String.equal a b)
+  | NativeString a, NativeString b -> Some (String.equal a b)
   | Tuple (ta, a, _), Tuple (tb, b, _) ->
       if ta <> tb || Array.length a <> Array.length b
       then Some false
@@ -286,21 +286,21 @@ let rec constant_equal a b =
   | Float_array a, Float_array b -> Some (Array.equal Float.equal a b)
   | Int a, Int b -> Some (Int32.equal a b)
   | Float a, Float b -> Some (Float.equal a b)
-  | String _, IString _ | IString _, String _ -> None
+  | String _, NativeString _ | NativeString _, String _ -> None
   | Int _, Float _ | Float _, Int _ -> None
   | Tuple ((0 | 254), _, _), Float_array _ -> None
   | Float_array _, Tuple ((0 | 254), _, _) -> None
-  | Tuple _, (String _ | IString _ | Int64 _ | Int _ | Float _ | Float_array _) ->
+  | Tuple _, (String _ | NativeString _ | Int64 _ | Int _ | Float _ | Float_array _) ->
       Some false
-  | Float_array _, (String _ | IString _ | Int64 _ | Int _ | Float _ | Tuple _) ->
+  | Float_array _, (String _ | NativeString _ | Int64 _ | Int _ | Float _ | Tuple _) ->
       Some false
   | String _, (Int64 _ | Int _ | Float _ | Tuple _ | Float_array _) -> Some false
-  | IString _, (Int64 _ | Int _ | Float _ | Tuple _ | Float_array _) -> Some false
-  | Int64 _, (String _ | IString _ | Int _ | Float _ | Tuple _ | Float_array _) ->
+  | NativeString _, (Int64 _ | Int _ | Float _ | Tuple _ | Float_array _) -> Some false
+  | Int64 _, (String _ | NativeString _ | Int _ | Float _ | Tuple _ | Float_array _) ->
       Some false
-  | Float _, (String _ | IString _ | Float_array _ | Int64 _ | Tuple (_, _, _)) ->
+  | Float _, (String _ | NativeString _ | Float_array _ | Int64 _ | Tuple (_, _, _)) ->
       Some false
-  | Int _, (String _ | IString _ | Float_array _ | Int64 _ | Tuple (_, _, _)) ->
+  | Int _, (String _ | NativeString _ | Float_array _ | Int64 _ | Tuple (_, _, _)) ->
       Some false
 
 type prim_arg =
@@ -360,7 +360,7 @@ module Print = struct
   let rec constant f x =
     match x with
     | String s -> Format.fprintf f "%S" s
-    | IString s -> Format.fprintf f "%S" s
+    | NativeString s -> Format.fprintf f "%Sj" s
     | Float fl -> Format.fprintf f "%.12g" fl
     | Float_array a ->
         Format.fprintf f "[|";
@@ -531,13 +531,25 @@ let prepend ({ start; blocks; free_pc } as p) body =
       let free_pc = free_pc + 1 in
       { start = new_start; blocks; free_pc }
 
+let empty_block = { params = []; handler = None; body = []; branch = Stop }
+
 let empty =
   let start = 0 in
-  let free_pc = 1 in
-  let blocks =
-    Addr.Map.singleton start { params = []; handler = None; body = []; branch = Stop }
-  in
-  { start; blocks; free_pc }
+  let blocks = Addr.Map.singleton start empty_block in
+  { start; blocks; free_pc = start + 1 }
+
+let is_empty p =
+  match Addr.Map.cardinal p.blocks with
+  | 0 -> true
+  | 1 -> (
+      let _, v = Addr.Map.choose p.blocks in
+      match v with
+      | { handler = None; body; branch = Stop; params = _ } -> (
+          match body with
+          | ([] | [ Let (_, Prim (Extern "caml_get_global_data", _)) ]) when true -> true
+          | _ -> false)
+      | _ -> false)
+  | _ -> false
 
 let fold_children blocks pc f accu =
   let block = Addr.Map.find pc blocks in
@@ -601,9 +613,10 @@ let with_invariant = Debug.find "invariant"
 
 let check_defs = false
 
-let invariant { blocks; _ } =
+let invariant { blocks; start; _ } =
   if with_invariant ()
-  then
+  then (
+    assert (Addr.Map.mem start blocks);
     let defs = Array.make (Var.count ()) false in
     let check_cont (cont, args) =
       let b = Addr.Map.find cont blocks in
@@ -655,4 +668,4 @@ let invariant { blocks; _ } =
         Option.iter block.handler ~f:(fun (_, cont) -> check_cont cont);
         List.iter block.body ~f:check_instr;
         check_last block.branch)
-      blocks
+      blocks)
