@@ -18,4 +18,8 @@
 
 open! Stdlib
 
-let global_object = "joo_global_object"
+let global_object = "globalThis"
+
+let old_global_object = "joo_global_object"
+
+let exports = "jsoo_exports"
