@@ -23,6 +23,10 @@ let debug = Debug.find "main"
 
 let times = Debug.find "times"
 
+let should_export = function
+  | `Iife -> false
+  | `Named _ | `Anonymous -> true
+
 let tailcall p =
   if debug () then Format.eprintf "Tail-call optimization...@.";
   Tailcall.f p
@@ -140,9 +144,10 @@ let round2 = flow +> specialize' +> eval +> deadcode +> o1
 
 let o3 = loop 10 "tailcall+inline" round1 1 +> loop 10 "flow" round2 1 +> print
 
-let generate d ~exported_runtime (p, live_vars) =
+let generate d ~exported_runtime ~wrap_with_fun (p, live_vars) =
   if times () then Format.eprintf "Start Generation...@.";
-  Generate.f p ~exported_runtime ~live_vars d
+  let should_export = should_export wrap_with_fun in
+  Generate.f p ~exported_runtime ~live_vars ~should_export d
 
 let header formatter ~custom_header =
   (match custom_header with
@@ -165,16 +170,8 @@ let extra_js_files =
          try
            let name = Builtins.File.name file in
            let ss =
-             List.fold_left
-               (Linker.parse_builtin file)
-               ~init:StringSet.empty
-               ~f:(fun ss frag ->
-                 match frag with
-                 | `Always_include _ -> ss
-                 | `Some { Linker.provides; _ } -> (
-                     match provides with
-                     | Some (_, name, _, _) -> StringSet.add name ss
-                     | _ -> ss))
+             List.concat_map ~f:Linker.Fragment.provides (Linker.parse_builtin file)
+             |> StringSet.of_list
            in
            (name, ss) :: acc
          with _ -> acc))
@@ -326,17 +323,17 @@ let coloring js =
   if times () then Format.eprintf "  coloring: %a@." Timer.print t;
   js
 
-let output formatter ~standalone ~custom_header ?source_map () js =
+let output formatter ~standalone ~custom_header ~source_map () js =
   let t = Timer.make () in
   if times () then Format.eprintf "Start Writing file...@.";
   if standalone then header ~custom_header formatter;
   Js_output.program formatter ?source_map js;
   if times () then Format.eprintf "  write: %a@." Timer.print t
 
-let pack ~global { Linker.runtime_code = js; always_required_codes } =
+let pack ~wrap_with_fun ~standalone { Linker.runtime_code = js; always_required_codes } =
   let module J = Javascript in
   let t = Timer.make () in
-  if times () then Format.eprintf "Start Flagizing js...@.";
+  if times () then Format.eprintf "Start Optimizing js...@.";
   (* pre pack optim *)
   let js =
     if Config.Flag.share_constant ()
@@ -357,39 +354,53 @@ let pack ~global { Linker.runtime_code = js; always_required_codes } =
     else js
   in
   (* pack *)
-  let use_strict js ~can_use_strict =
-    if Config.Flag.strictmode () && can_use_strict
-    then (J.Statement (J.Expression_statement (J.EStr ("use strict", `Utf8))), J.N) :: js
-    else js
-  in
-  let wrap_in_iifa ~can_use_strict js =
-    let f =
-      J.EFun (None, [ J.ident global_object ], use_strict js ~can_use_strict, J.U)
+  let wrap_in_iife ~use_strict js =
+    let var ident e =
+      J.Statement (J.Variable_statement [ J.ident ident, Some (e, J.N) ]), J.N
     in
-    let expr =
-      match global with
-      | `Function -> f
-      | `Bind_to _ -> f
-      | `Custom name -> J.ECall (f, [ J.EVar (J.ident name), `Not_spread ], J.N)
-      | `Auto ->
-          let global =
-            J.ECall
-              ( J.EFun
-                  ( None
-                  , []
-                  , [ ( J.Statement (J.Return_statement (Some (J.EVar (J.ident "this"))))
-                      , J.N )
-                    ]
-                  , J.N )
-              , []
-              , J.N )
+    let expr e = J.Statement (J.Expression_statement e), J.N in
+    let freenames =
+      let o = new Js_traverse.free in
+      let (_ : J.program) = o#program js in
+      o#get_free_name
+    in
+    let export_shim js =
+      if StringSet.mem Constant.exports freenames
+      then
+        if should_export wrap_with_fun
+        then var Constant.exports (J.EObj []) :: js
+        else
+          let export_node =
+            let s =
+              Printf.sprintf
+                {|((typeof module === 'object' && module.exports) || %s)|}
+                global_object
+            in
+            let lex = Parse_js.Lexer.of_lexbuf (Lexing.from_string s) in
+            Parse_js.parse_expr lex
           in
-          J.ECall (f, [ global, `Not_spread ], J.N)
+          var Constant.exports export_node :: js
+      else js
     in
-    match global with
-    | `Bind_to name ->
-        [ J.Statement (J.Variable_statement [ J.ident name, Some (expr, J.N) ]), J.N ]
-    | _ -> [ J.Statement (J.Expression_statement expr), J.N ]
+    let old_global_object_shim js =
+      if StringSet.mem Constant.old_global_object freenames
+      then var Constant.old_global_object (J.EVar (J.ident global_object)) :: js
+      else js
+    in
+
+    let efun args body = J.EFun (None, args, body, J.U) in
+    let sfun name args body = J.Function_declaration (name, args, body, J.U), J.U in
+    let mk f =
+      let js = export_shim js in
+      let js = old_global_object_shim js in
+      let js = if use_strict then expr (J.EStr ("use strict", `Utf8)) :: js else js in
+      f [ J.ident global_object ] js
+    in
+    match wrap_with_fun with
+    | `Anonymous -> expr (mk efun)
+    | `Named name -> mk (sfun (J.ident name))
+    | `Iife ->
+        expr (J.ECall (mk efun, [ J.EVar (J.ident global_object), `Not_spread ], J.N))
   in
   let always_required_js =
     (* consider adding a comments in the generated file with original
@@ -401,10 +412,55 @@ let pack ~global { Linker.runtime_code = js; always_required_codes } =
     List.map
       always_required_codes
       ~f:(fun { Linker.program; filename = _; requires = _ } ->
-        wrap_in_iifa ~can_use_strict:false program)
+        wrap_in_iife ~use_strict:false program)
+  in
+  let runtime_js = wrap_in_iife ~use_strict:(Config.Flag.strictmode ()) js in
+  let js = always_required_js @ [ runtime_js ] in
+  let js =
+    match wrap_with_fun, standalone with
+    | `Named name, (true | false) ->
+        let export_node =
+          let s =
+            Printf.sprintf
+              {|
+if (typeof module === 'object' && module.exports) {
+  module['exports'] = %s;
+}
+|}
+              name
+          in
+          let lex = Parse_js.Lexer.of_lexbuf (Lexing.from_string s) in
+          Parse_js.parse lex
+        in
+        js @ export_node
+    | `Anonymous, _ -> js
+    | `Iife, false -> js
+    | `Iife, true ->
+        let e =
+          let s =
+            {|
+(function (Object) {
+  typeof globalThis !== 'object' && (
+    this ?
+      get() :
+      (Object.defineProperty(Object.prototype, '_T_', {
+        configurable: true,
+        get: get
+      }), _T_)
+  );
+  function get() {
+    var global = this || self;
+    global.globalThis = global;
+    delete Object.prototype._T_;
+  }
+}(Object));
+|}
+          in
+          let lex = Parse_js.Lexer.of_lexbuf (Lexing.from_string s) in
+          Parse_js.parse lex
+        in
+        e @ js
   in
-  let runtime_js = wrap_in_iifa ~can_use_strict:true js in
-  let js = List.flatten always_required_js @ runtime_js in
   (* post pack optim *)
   let t3 = Timer.make () in
   let js = (new Js_traverse.simpl)#program js in
@@ -434,33 +490,77 @@ let configure formatter p =
 
 type profile = Code.program -> Code.program
 
+let full
+    ~standalone
+    ~wrap_with_fun
+    ~profile
+    ~dynlink
+    ~linkall
+    ~source_map
+    ~custom_header
+    formatter
+    d
+    p =
+  let exported_runtime = not standalone in
+  let linkall = linkall || dynlink in
+
+  let opt =
+    configure formatter
+    +> specialize_js_once
+    +> profile
+    +> Generate_closure.f
+    +> deadcode'
+  in
+  let emit =
+    generate d ~exported_runtime ~wrap_with_fun
+    +> link ~standalone ~linkall ~export_runtime:dynlink
+    +> pack ~wrap_with_fun ~standalone
+    +> coloring
+    +> check_js
+    +> output formatter ~standalone ~custom_header ~source_map ()
+  in
+  if times () then Format.eprintf "Start Optimizing...@.";
+  let t = Timer.make () in
+  let r = opt p in
+  let () = if times () then Format.eprintf " optimizations : %a@." Timer.print t in
+  emit r
+
 let f
     ?(standalone = true)
-    ?(global = `Auto)
+    ?(wrap_with_fun = `Iife)
     ?(profile = o1)
     ?(dynlink = false)
     ?(linkall = false)
     ?source_map
     ?custom_header
     formatter
-    d =
-  let exported_runtime = not standalone in
-  let linkall = linkall || dynlink in
-  configure formatter
-  +> specialize_js_once
-  +> profile
-  +> Generate_closure.f
-  +> deadcode'
-  +> generate d ~exported_runtime
-  +> link ~standalone ~linkall ~export_runtime:dynlink
-  +> pack ~global
-  +> coloring
-  +> check_js
-  +> output formatter ~standalone ~custom_header ?source_map ()
+    d
+    p =
+  full
+    ~standalone
+    ~wrap_with_fun
+    ~profile
+    ~dynlink
+    ~linkall
+    ~source_map
+    ~custom_header
+    formatter
+    d
+    p
 
 let from_string prims s formatter =
   let p, d = Parse_bytecode.from_string prims s in
-  f ~standalone:false ~global:`Function formatter d p
+  full
+    ~standalone:false
+    ~wrap_with_fun:`Anonymous
+    ~profile:o1
+    ~dynlink:false
+    ~linkall:false
+    ~source_map:None
+    ~custom_header:None
+    formatter
+    d
+    p
 
 let profiles = [ 1, o1; 2, o2; 3, o3 ]
 
