@@ -20,52 +20,40 @@
 
 open! Stdlib
 
-type fragment_ =
-  { provides :
-      (Parse_info.t option * string * Primitive.kind * Primitive.kind_arg list option)
-      option
-  ; requires : string list
-  ; version_constraint : ((int -> int -> bool) * string) list list
-  ; weakdef : bool
-  ; always : bool
-  ; code : Javascript.program
-  ; ignore : [ `No | `Because of Primitive.condition ]
-  }
+module Fragment = struct
+  type fragment_ =
+    { provides :
+        (Parse_info.t * string * Primitive.kind * Primitive.kind_arg list option) option
+    ; requires : string list
+    ; version_constraint : ((int -> int -> bool) * string) list list
+    ; weakdef : bool
+    ; always : bool
+    ; code : Javascript.program
+    ; js_string : bool option
+    ; fragment_target : Target_env.t option
+    }
 
-type fragment =
-  [ `Always_include of Javascript.program
-  | `Some of fragment_
-  ]
+  type t =
+    [ `Always_include of Javascript.program
+    | `Some of fragment_
+    ]
+
+  let provides = function
+    | `Always_include _ -> []
+    | `Some { provides = Some (_, name, _, _); _ } -> [ name ]
+    | `Some _ -> []
+end
 
 let loc pi =
   match pi with
-  | Some { Parse_info.src = Some src; line; _ }
-  | Some { Parse_info.name = Some src; line; _ } ->
+  | { Parse_info.src = Some src; line; _ } | { Parse_info.name = Some src; line; _ } ->
       Printf.sprintf "%s:%d" src line
-  | None | Some _ -> "unknown location"
-
-let parse_annot loc s =
-  match String.drop_prefix ~prefix:"//" s with
-  | None -> None
-  | Some s -> (
-      let buf = Lexing.from_string s in
-      try
-        match Annot_parser.annot Annot_lexer.main buf with
-        | `Requires (_, l) -> Some (`Requires (Some loc, l))
-        | `Provides (_, n, k, ka) -> Some (`Provides (Some loc, n, k, ka))
-        | `Version (_, l) -> Some (`Version (Some loc, l))
-        | `Weakdef _ -> Some (`Weakdef (Some loc))
-        | `Always _ -> Some (`Always (Some loc))
-        | `If (_, name) -> Some (`If (Some loc, name))
-        | `Ifnot (_, name) -> Some (`Ifnot (Some loc, name))
-      with
-      | Not_found -> None
-      | _ -> None)
+  | _ -> "unknown location"
 
 let error s = Format.ksprintf (fun s -> failwith s) s
 
 let parse_from_lex ~filename lex =
-  let program, _prev, comments =
+  let program, _ =
     try Parse_js.parse' lex
     with Parse_js.Parsing_error pi ->
       let name =
@@ -80,105 +68,68 @@ let parse_from_lex ~filename lex =
         pi.Parse_info.line
         pi.Parse_info.col
   in
-  let rec take_annot_before loc acc = function
-    | [] -> acc, []
-    | x :: l ->
-        if (Js_token.info x).Parse_info.idx <= loc.Parse_info.idx
-        then
-          let acc =
-            match x with
-            | Js_token.TComment (str, info) -> (
-                match parse_annot info str with
-                | None -> acc
-                | Some a -> a :: acc)
-            | Js_token.TCommentLineDirective (_, _) -> []
-            | _ -> acc
-          in
-          take_annot_before loc acc l
-        else acc, x :: l
+  let rec collect_without_annot acc = function
+    | [] -> List.rev acc, []
+    | (x, []) :: program -> collect_without_annot (x :: acc) program
+    | (_, _ :: _) :: _ as program -> List.rev acc, program
   in
-  let status, blocks, _comments =
-    List.fold_left
-      program
-      ~init:(`Annot [], [], comments)
-      ~f:(fun (status, blocks, comments) t ->
-        match t with
-        | _, Javascript.Pi loc ->
-            let a, rest = take_annot_before loc [] comments in
-            let status, blocks =
-              match a, status with
-              | [], `Code (annot, code) -> `Code (annot, t :: code), blocks
-              | annot1, `Annot annot2 -> `Code (annot1 @ annot2, [ t ]), blocks
-              | annot1, `Code (annot2, code2) ->
-                  `Code (annot1, [ t ]), (List.rev annot2, List.rev code2) :: blocks
-            in
-            status, blocks, rest
-        | _, Javascript.N ->
-            (* FIXME: This is not correct *)
-            let status, blocks =
-              match status with
-              | `Code (annot, code) -> `Code (annot, t :: code), blocks
-              | `Annot annot -> `Code (annot, [ t ]), blocks
-            in
-            status, blocks, comments
-        | _, Javascript.U -> assert false)
-  in
-  let blocks =
-    match status with
-    | `Annot _ -> blocks
-    | `Code (annot, code) -> (List.rev annot, List.rev code) :: blocks
+  let rec collect acc program =
+    match program with
+    | [] -> List.rev acc
+    | (x, []) :: program ->
+        let code, program = collect_without_annot [ x ] program in
+        collect (([], code) :: acc) program
+    | (x, annots) :: program ->
+        let code, program = collect_without_annot [ x ] program in
+        collect ((annots, code) :: acc) program
   in
+  let blocks = collect [] program in
   let res =
     List.rev_map blocks ~f:(fun (annot, code) ->
         match annot with
         | [] -> `Always_include code
         | annot ->
-            let no_annot_fragment =
+            let initial_fragment : Fragment.fragment_ =
               { provides = None
               ; requires = []
               ; version_constraint = []
               ; weakdef = false
               ; always = false
               ; code
-              ; ignore = `No
+              ; js_string = None
+              ; fragment_target = None
               }
             in
             let fragment =
-              List.fold_left annot ~init:no_annot_fragment ~f:(fun fragment a ->
+              List.fold_left
+                annot
+                ~init:initial_fragment
+                ~f:(fun (fragment : Fragment.fragment_) (_, pi, a) ->
                   match a with
-                  | `Provides (pi, name, kind, ka) ->
+                  | `Provides (name, kind, ka) ->
                       { fragment with provides = Some (pi, name, kind, ka) }
-                  | `Requires (_, mn) ->
-                      { fragment with requires = mn @ fragment.requires }
-                  | `Version (_, l) ->
+                  | `Requires mn -> { fragment with requires = mn @ fragment.requires }
+                  | `Version l ->
                       { fragment with
                         version_constraint = l :: fragment.version_constraint
                       }
-                  | `Weakdef _ -> { fragment with weakdef = true }
-                  | `Always _ -> { fragment with always = true }
-                  | `If (_, "js-string") as reason ->
-                      if not (Config.Flag.use_js_string ())
-                      then { fragment with ignore = `Because reason }
-                      else fragment
-                  | `Ifnot (_, "js-string") as reason ->
-                      if Config.Flag.use_js_string ()
-                      then { fragment with ignore = `Because reason }
-                      else fragment
-                  | `If (pi, name) | `Ifnot (pi, name) ->
-                      let loc =
-                        match pi with
-                        | None -> ""
-                        | Some loc ->
-                            Format.sprintf "%d:%d" loc.Parse_info.line loc.Parse_info.col
+                  | `Weakdef -> { fragment with weakdef = true }
+                  | `Always -> { fragment with always = true }
+                  | (`Ifnot "js-string" | `If "js-string") as i ->
+                      let b =
+                        match i with
+                        | `If _ -> true
+                        | `Ifnot _ -> false
                       in
-                      let filename =
-                        match pi with
-                        | Some { Parse_info.src = Some x; _ }
-                        | Some { Parse_info.name = Some x; _ } ->
-                            x
-                        | _ -> "??"
-                      in
-                      Format.eprintf "Unkown flag %S in %s %s\n" name filename loc;
+                      if Option.is_some fragment.js_string
+                      then Format.eprintf "Duplicated js-string in %s\n" (loc pi);
+                      { fragment with js_string = Some b }
+                  | `If name when Option.is_some (Target_env.of_string name) ->
+                      if Option.is_some fragment.fragment_target
+                      then Format.eprintf "Duplicated target_env in %s\n" (loc pi);
+                      { fragment with fragment_target = Target_env.of_string name }
+                  | `If name | `Ifnot name ->
+                      Format.eprintf "Unkown flag %S in %s\n" name (loc pi);
                       fragment)
             in
             `Some fragment)
@@ -292,6 +243,16 @@ let check_primitive ~name pi ~code ~requires =
   let freename = StringSet.diff freename Reserved.keyword in
   let freename = StringSet.diff freename Reserved.provided in
   let freename = StringSet.remove Constant.global_object freename in
+  if StringSet.mem Constant.old_global_object freename && false
+     (* Don't warn yet, we want to give a transition period where both
+        "globalThis" and "joo_global_object" are allowed without extra
+        noise *)
+  then
+    warn
+      "warning: %s: 'joo_global_object' is being deprecated, please use `globalThis` \
+       instead@."
+      (loc pi);
+  let freename = StringSet.remove Constant.old_global_object freename in
   if not (StringSet.mem name free#get_def_name)
   then
     warn
@@ -323,15 +284,29 @@ type output =
   ; always_required_codes : always_required list
   }
 
+type provided =
+  { id : int
+  ; pi : Parse_info.t
+  ; weakdef : bool
+  ; target_env : Target_env.t
+  }
+
 let last_code_id = ref 0
 
+let always_included = ref []
+
 let provided = Hashtbl.create 31
 
 let provided_rev = Hashtbl.create 31
 
 let code_pieces = Hashtbl.create 31
 
-let always_included = ref []
+let reset () =
+  last_code_id := 0;
+  always_included := [];
+  Hashtbl.clear provided;
+  Hashtbl.clear provided_rev;
+  Hashtbl.clear code_pieces
 
 class traverse_and_find_named_values all =
   object
@@ -342,8 +317,7 @@ class traverse_and_find_named_values all =
       (match x with
       | ECall
           (EVar (S { name = "caml_named_value"; _ }), [ (EStr (v, _), `Not_spread) ], _)
-        ->
-          all := StringSet.add v !all
+        -> all := StringSet.add v !all
       | _ -> ());
       self#expression x
   end
@@ -354,64 +328,121 @@ let find_named_value code =
   ignore (p#program code);
   !all
 
-let load_fragment ~filename f =
+let load_fragment ~target_env ~filename (f : Fragment.t) =
   match f with
   | `Always_include code ->
-      always_included := { filename; program = code; requires = [] } :: !always_included
-  | `Some { provides; requires; version_constraint; weakdef; always; code; ignore } -> (
-      match ignore with
-      | `Because _ -> ()
-      | `No ->
-          let vmatch =
-            match version_constraint with
-            | [] -> true
-            | l -> List.exists l ~f:version_match
-          in
-          if vmatch
-          then (
-            incr last_code_id;
-            let id = !last_code_id in
-            match provides with
-            | None ->
-                if always
-                then
-                  always_included :=
-                    { filename; program = code; requires } :: !always_included
-                else
-                  error
-                    "Found JavaScript code with neither `//Provides` nor `//Always` in \
-                     file %S@."
-                    filename
-            | Some (pi, name, kind, ka) ->
-                let code = Macro.f code in
-                let module J = Javascript in
-                let rec find = function
-                  | [] -> None
-                  | (J.Function_declaration (J.S { J.name = n; _ }, l, _, _), _) :: _
-                    when String.equal name n ->
-                      Some (List.length l)
-                  | _ :: rem -> find rem
-                in
-                let arity = find code in
-                let named_values = find_named_value code in
-                Primitive.register name kind ka arity;
-                StringSet.iter Primitive.register_named_value named_values;
-                (if Hashtbl.mem provided name
-                then
-                  let _, ploc, weakdef = Hashtbl.find provided name in
-                  if not weakdef
-                  then
+      always_included := { filename; program = code; requires = [] } :: !always_included;
+      `Ok
+  | `Some
+      { provides
+      ; requires
+      ; version_constraint
+      ; weakdef
+      ; always
+      ; code
+      ; js_string
+      ; fragment_target
+      } -> (
+      let ignore_because_of_js_string =
+        match js_string, Config.Flag.use_js_string () with
+        | Some true, false | Some false, true -> true
+        | None, _ | Some true, true | Some false, false -> false
+      in
+      let ignore_because_of_version_constraint =
+        match version_constraint with
+        | [] -> false
+        | l -> not (List.exists l ~f:version_match)
+      in
+      if ignore_because_of_version_constraint || ignore_because_of_js_string
+      then `Ignored
+      else
+        match provides with
+        | None ->
+            if always
+            then (
+              always_included :=
+                { filename; program = code; requires } :: !always_included;
+              `Ok)
+            else
+              error
+                "Found JavaScript code with neither `//Provides` nor `//Always` in file \
+                 %S@."
+                filename
+        | Some (pi, name, kind, ka) ->
+            let fragment_target =
+              Option.value ~default:Target_env.Isomorphic fragment_target
+            in
+            let exists =
+              try `Exists (Hashtbl.find provided name) with Not_found -> `New
+            in
+            let is_updating =
+              match
+                exists, (target_env : Target_env.t), (fragment_target : Target_env.t)
+              with
+              (* permit default, un-annotated symbols *)
+              | `New, _, Isomorphic -> true
+              (* permit env specializations *)
+              | `New, Nodejs, Nodejs
+              | `New, Browser, Browser
+              | `Exists { target_env = Isomorphic; _ }, Nodejs, Nodejs
+              | `Exists { target_env = Isomorphic; _ }, Browser, Browser -> true
+              (* ignore non target matched envs *)
+              | (`Exists _ | `New), Isomorphic, (Browser | Nodejs)
+              | (`Exists _ | `New), Browser, Nodejs
+              | (`Exists _ | `New), Nodejs, Browser -> false
+              (* Ignore env unspecializations *)
+              | `Exists { target_env = Nodejs; _ }, Nodejs, Isomorphic
+              | `Exists { target_env = Browser; _ }, Browser, Isomorphic -> false
+              (* The following are impossible *)
+              | `Exists { target_env = Nodejs; _ }, Browser, _
+              | `Exists { target_env = Browser; _ }, Nodejs, _
+              | `Exists { target_env = Nodejs | Browser; _ }, Isomorphic, _ ->
+                  assert false
+              (* collision detected *)
+              | `Exists ({ target_env = Nodejs; _ } as p), Nodejs, Nodejs
+              | `Exists ({ target_env = Isomorphic; _ } as p), Nodejs, Isomorphic
+              | `Exists ({ target_env = Browser; _ } as p), Browser, Browser
+              | `Exists ({ target_env = Isomorphic; _ } as p), Browser, Isomorphic
+              | `Exists ({ target_env = Isomorphic; _ } as p), Isomorphic, Isomorphic ->
+                  if p.weakdef
+                  then true
+                  else (
                     warn
                       "warning: overriding primitive %S\n  old: %s\n  new: %s@."
                       name
-                      (loc ploc)
-                      (loc pi));
-                Hashtbl.add provided name (id, pi, weakdef);
-                Hashtbl.add provided_rev id (name, pi);
-                check_primitive ~name pi ~code ~requires;
-                Hashtbl.add code_pieces id (code, requires)))
+                      (loc p.pi)
+                      (loc pi);
+                    true)
+            in
+            if not is_updating
+            then `Ignored
+            else
+              let rec find_arity = function
+                | [] -> None
+                | ( Javascript.Function_declaration
+                      (Javascript.S { Javascript.name = n; _ }, l, _, _)
+                  , _ )
+                  :: _
+                  when String.equal name n -> Some (List.length l)
+                | _ :: rem -> find_arity rem
+              in
+              incr last_code_id;
+              let id = !last_code_id in
+              let code = Macro.f code in
+              let arity = find_arity code in
+              let named_values = find_named_value code in
+              check_primitive ~name pi ~code ~requires;
+              Primitive.register name kind ka arity;
+              StringSet.iter Primitive.register_named_value named_values;
+              Hashtbl.add provided name { id; pi; weakdef; target_env = fragment_target };
+              Hashtbl.add provided_rev id (name, pi);
+              Hashtbl.add code_pieces id (code, requires);
+              `Ok)
 
-let add_file filename = List.iter (parse_file filename) ~f:(load_fragment ~filename)
+let add_file ~target_env filename =
+  List.iter (parse_file filename) ~f:(fun frag ->
+      let (`Ok | `Ignored) = load_fragment ~target_env ~filename frag in
+      ())
 
 let get_provided () =
   Hashtbl.fold (fun k _ acc -> StringSet.add k acc) provided StringSet.empty
@@ -441,16 +472,22 @@ let check_deps () =
           ())
     code_pieces
 
-let load_files l =
-  List.iter l ~f:add_file;
+let load_fragments ~target_env ~filename l =
+  List.iter l ~f:(fun frag ->
+      let (`Ok | `Ignored) = load_fragment ~target_env ~filename frag in
+      ());
+  check_deps ()
+
+let load_files ~target_env l =
+  List.iter l ~f:(fun filename -> add_file ~target_env filename);
   check_deps ()
 
 (* resolve *)
 let rec resolve_dep_name_rev visited path nm =
   let id =
     try
-      let x, _, _ = Hashtbl.find provided nm in
-      x
+      let x = Hashtbl.find provided nm in
+      x.id
     with Not_found -> error "missing dependency '%s'@." nm
   in
   resolve_dep_id_rev visited path id
@@ -488,7 +525,7 @@ let resolve_deps ?(linkall = false) visited_rev used =
       (* link all primitives *)
       let prog, set =
         Hashtbl.fold
-          (fun nm (_id, _, _) (visited, set) ->
+          (fun nm _ (visited, set) ->
             resolve_dep_name_rev visited [] nm, StringSet.add nm set)
           provided
           (visited_rev, StringSet.empty)
@@ -539,6 +576,6 @@ let all state =
 
 let origin ~name =
   try
-    let _, ploc, _ = Hashtbl.find provided name in
-    Option.bind ploc ~f:(fun ploc -> ploc.Parse_info.src)
+    let x = Hashtbl.find provided name in
+    x.pi.Parse_info.src
   with Not_found -> None
