@@ -29,6 +29,15 @@ open Js
 
 class type cssStyleDeclaration =
   object
+    method setProperty :
+      js_string t -> js_string t -> js_string t optdef -> js_string t meth
+
+    method getPropertyValue : js_string t -> js_string t meth
+
+    method getPropertyPriority : js_string t -> js_string t meth
+
+    method removeProperty : js_string t -> js_string t meth
+
     method animation : js_string t prop
 
     method animationDelay : js_string t prop
@@ -631,6 +640,15 @@ and mediaEvent =
     inherit event
   end
 
+and messageEvent =
+  object
+    inherit event
+
+    method data : Unsafe.any opt readonly_prop
+
+    method source : Unsafe.any opt readonly_prop
+  end
+
 (** {2 HTML elements} *)
 
 and nodeSelector =
@@ -2045,7 +2063,7 @@ class type navigator =
 
     method userLanguage : js_string t optdef readonly_prop
 
-    method maxTouchPoints : int t readonly_prop
+    method maxTouchPoints : int readonly_prop
   end
 
 class type screen =
@@ -2171,13 +2189,13 @@ class type window =
 
     method screen : screen t readonly_prop
 
-    method innerWidth : int optdef readonly_prop
+    method innerWidth : int readonly_prop
 
-    method innerHeight : int optdef readonly_prop
+    method innerHeight : int readonly_prop
 
-    method outerWidth : int optdef readonly_prop
+    method outerWidth : int readonly_prop
 
-    method outerHeight : int optdef readonly_prop
+    method outerHeight : int readonly_prop
 
     method getComputedStyle : #element t -> cssStyleDeclaration t meth
 
@@ -2439,6 +2457,8 @@ module Event : sig
 
   val lostpointercapture : pointerEvent t typ
 
+  val message : messageEvent t typ
+
   val pause : mediaEvent t typ
 
   val play : mediaEvent t typ
@@ -2972,6 +2992,7 @@ val opt_tagged : #element t opt -> taggedElement option
 type taggedEvent =
   | MouseEvent of mouseEvent t
   | KeyboardEvent of keyboardEvent t
+  | MessageEvent of messageEvent t
   | MousewheelEvent of mousewheelEvent t
   | MouseScrollEvent of mouseScrollEvent t
   | PopStateEvent of popStateEvent t
@@ -3123,6 +3144,8 @@ module CoerceTo : sig
   val mouseScrollEvent : #event t -> mouseScrollEvent t opt
 
   val popStateEvent : #event t -> popStateEvent t opt
+
+  val messageEvent : #event t -> messageEvent t opt
 end
 
 type timeout_id_safe
@@ -3139,6 +3162,7 @@ val js_array_of_collection : #element collection Js.t -> #element Js.t Js.js_arr
 (** {2 Deprecated function.} *)
 
 val _requestAnimationFrame : (unit -> unit) Js.callback -> unit
+  [@@ocaml.deprecated "[since 2.6] Use [Dom_html.window##requestAnimationFrame] instead."]
 (** Call the appropriate [requestAnimationFrame] method variant
       (depending on the navigator), or sleep for a short amount
       of time when there no such method is provided. We currently
