@@ -64,7 +64,7 @@ module Js = struct
 
     external pure_js_expr : string -> 'a = "caml_pure_js_expr"
 
-    let global = pure_js_expr "joo_global_object"
+    let global = pure_js_expr "globalThis"
 
     external callback : ('a -> 'b) -> ('c, 'a -> 'b) meth_callback = "%identity"
 
@@ -214,106 +214,112 @@ module Js = struct
 
   external wrap_meth_callback : ('a -> 'b) -> ('a, 'b) meth_callback
     = "caml_js_wrap_meth_callback"
+
+  (****)
+
+  let _true = Unsafe.pure_js_expr "true"
+
+  let _false = Unsafe.pure_js_expr "false"
+
+  type match_result_handle
+
+  type string_array
+
+  class type js_string =
+    object
+      method toString : js_string t meth
+
+      method valueOf : js_string t meth
+
+      method charAt : int -> js_string t meth
+
+      method charCodeAt : int -> float meth
+
+      (* This may return NaN... *)
+      method concat : js_string t -> js_string t meth
+
+      method concat_2 : js_string t -> js_string t -> js_string t meth
+
+      method concat_3 : js_string t -> js_string t -> js_string t -> js_string t meth
+
+      method concat_4 :
+        js_string t -> js_string t -> js_string t -> js_string t -> js_string t meth
+
+      method indexOf : js_string t -> int meth
+
+      method indexOf_from : js_string t -> int -> int meth
+
+      method lastIndexOf : js_string t -> int meth
+
+      method lastIndexOf_from : js_string t -> int -> int meth
+
+      method localeCompare : js_string t -> float meth
+
+      method _match : regExp t -> match_result_handle t opt meth
+
+      method replace : regExp t -> js_string t -> js_string t meth
+
+      method replace_string : js_string t -> js_string t -> js_string t meth
+
+      method search : regExp t -> int meth
+
+      method slice : int -> int -> js_string t meth
+
+      method slice_end : int -> js_string t meth
+
+      method split : js_string t -> string_array t meth
+
+      method split_limited : js_string t -> int -> string_array t meth
+
+      method split_regExp : regExp t -> string_array t meth
+
+      method split_regExpLimited : regExp t -> int -> string_array t meth
+
+      method substring : int -> int -> js_string t meth
+
+      method substring_toEnd : int -> js_string t meth
+
+      method toLowerCase : js_string t meth
+
+      method toLocaleLowerCase : js_string t meth
+
+      method toUpperCase : js_string t meth
+
+      method toLocaleUpperCase : js_string t meth
+
+      method trim : js_string t meth
+
+      method length : int readonly_prop
+    end
+
+  and regExp =
+    object
+      method exec : js_string t -> match_result_handle t opt meth
+
+      method test : js_string t -> bool t meth
+
+      method toString : js_string t meth
+
+      method source : js_string t readonly_prop
+
+      method global : bool t readonly_prop
+
+      method ignoreCase : bool t readonly_prop
+
+      method multiline : bool t readonly_prop
+
+      method lastIndex : int prop
+    end
+
+  (* string is used by ppx_js, it needs to come before any use of the
+     new syntax in this file *)
+  external string : string -> js_string t = "caml_jsstring_of_string"
+
+  external to_string : js_string t -> string = "caml_string_of_jsstring"
 end
 
 include Js
 
-(****)
-
-let _true = Unsafe.pure_js_expr "true"
-
-let _false = Unsafe.pure_js_expr "false"
-
-type match_result_handle
-
-type string_array
-
-class type js_string =
-  object
-    method toString : js_string t meth
-
-    method valueOf : js_string t meth
-
-    method charAt : int -> js_string t meth
-
-    method charCodeAt : int -> float meth
-
-    (* This may return NaN... *)
-    method concat : js_string t -> js_string t meth
-
-    method concat_2 : js_string t -> js_string t -> js_string t meth
-
-    method concat_3 : js_string t -> js_string t -> js_string t -> js_string t meth
-
-    method concat_4 :
-      js_string t -> js_string t -> js_string t -> js_string t -> js_string t meth
-
-    method indexOf : js_string t -> int meth
-
-    method indexOf_from : js_string t -> int -> int meth
-
-    method lastIndexOf : js_string t -> int meth
-
-    method lastIndexOf_from : js_string t -> int -> int meth
-
-    method localeCompare : js_string t -> float meth
-
-    method _match : regExp t -> match_result_handle t opt meth
-
-    method replace : regExp t -> js_string t -> js_string t meth
-
-    method replace_string : js_string t -> js_string t -> js_string t meth
-
-    method search : regExp t -> int meth
-
-    method slice : int -> int -> js_string t meth
-
-    method slice_end : int -> js_string t meth
-
-    method split : js_string t -> string_array t meth
-
-    method split_limited : js_string t -> int -> string_array t meth
-
-    method split_regExp : regExp t -> string_array t meth
-
-    method split_regExpLimited : regExp t -> int -> string_array t meth
-
-    method substring : int -> int -> js_string t meth
-
-    method substring_toEnd : int -> js_string t meth
-
-    method toLowerCase : js_string t meth
-
-    method toLocaleLowerCase : js_string t meth
-
-    method toUpperCase : js_string t meth
-
-    method toLocaleUpperCase : js_string t meth
-
-    method trim : js_string t meth
-
-    method length : int readonly_prop
-  end
-
-and regExp =
-  object
-    method exec : js_string t -> match_result_handle t opt meth
-
-    method test : js_string t -> bool t meth
-
-    method toString : js_string t meth
-
-    method source : js_string t readonly_prop
-
-    method global : bool t readonly_prop
-
-    method ignoreCase : bool t readonly_prop
-
-    method multiline : bool t readonly_prop
-
-    method lastIndex : int prop
-  end
-
 class type string_constr =
   object
     method fromCharCode : int -> js_string t meth
@@ -665,15 +671,53 @@ class type error =
     method toString : js_string t meth
   end
 
-exception Error of error t
-
 let error_constr = Unsafe.global##._Error
 
-let _ = Callback.register_exception "jsError" (Error (Unsafe.obj [||]))
+module Js_error = struct
+  type error_t = error t
 
-let raise_js_error : error t -> 'a = Unsafe.js_expr "(function (exn) { throw exn })"
+  include Jsoo_runtime.Error
 
-external exn_with_js_backtrace : exn -> force:bool -> exn = "caml_exn_with_js_backtrace"
+  external of_error : error_t -> t = "%identity"
+
+  external to_error : t -> error_t = "%identity"
+
+  let name e = to_string (to_error e)##.name
+
+  let message e = to_string (to_error e)##.message
+
+  let stack (e : t) : string option =
+    Opt.to_option (Opt.map (to_error e)##.stack to_string)
+
+  let to_string e = to_string (to_error e)##toString
+end
+
+module Magic = struct
+  module type T = sig
+    exception Error of error t
+  end
+
+  type ('a, 'b) eq = Eq : ('a, 'a) eq
+
+  let (eq : (error t, Js_error.t) eq) = Obj.magic Eq
+
+  let m =
+    match eq with
+    | Eq ->
+        (module struct
+          exception Error = Js_error.Exn
+        end : T)
+
+  module Error = (val m : T)
+end
+
+include Magic.Error
+
+let raise_js_error e = Js_error.raise_ (Js_error.of_error e)
+
+let string_of_error e = Js_error.to_string (Js_error.of_error e)
+
+let exn_with_js_backtrace = Js_error.attach_js_backtrace
 
 external js_error_of_exn : exn -> error t opt = "caml_js_error_of_exception"
 
@@ -708,10 +752,6 @@ external bool : bool -> bool t = "caml_js_from_bool"
 
 external to_bool : bool t -> bool = "caml_js_to_bool"
 
-external string : string -> js_string t = "caml_jsstring_of_string"
-
-external to_string : js_string t -> string = "caml_string_of_jsstring"
-
 external array : 'a array -> 'a js_array t = "caml_js_from_array"
 
 external to_array : 'a js_array t -> 'a array = "caml_js_to_array"
@@ -737,7 +777,7 @@ let parseFloat (s : js_string t) : float =
 
 let _ =
   Printexc.register_printer (function
-      | Error e -> Some (to_string e##toString)
+      | Js_error.Exn e -> Some (Js_error.to_string e)
       | _ -> None)
 
 let _ =
@@ -745,11 +785,8 @@ let _ =
       let e : < .. > t = Obj.magic e in
       if instanceof e array_constructor then None else Some (to_string e##toString))
 
-let string_of_error e = to_string e##toString
-
-external get_export_var : unit -> < .. > t = "caml_js_export_var"
-
-let export_js (field : js_string t) x = Unsafe.set (get_export_var ()) field x
+let export_js (field : js_string t) x =
+  Unsafe.set (Unsafe.pure_js_expr "jsoo_exports") field x
 
 let export field x = export_js (string field) x
 
@@ -767,21 +804,3 @@ type float_prop = float prop
 external float : float -> float = "%identity"
 
 external to_float : float -> float = "%identity"
-
-[@@@ocaml.warning "-32-60"]
-
-module For_compatibility_only = struct
-  (* Add primitives for compatibility reasons. Existing users might
-     depend on it (e.g. gen_js_api), we dont want the ocaml compiler
-     to complain about theses missing primitives. *)
-
-  external caml_js_from_string : string -> js_string t = "caml_js_from_string"
-
-  external caml_js_to_byte_string : js_string t -> string = "caml_js_to_byte_string"
-
-  external caml_js_to_string : js_string t -> string = "caml_js_to_string"
-
-  external caml_list_of_js_array : 'a js_array t -> 'a list = "caml_list_of_js_array"
-
-  external caml_list_to_js_array : 'a list -> 'a js_array t = "caml_list_to_js_array"
-end
