@@ -25,9 +25,9 @@ function caml_trailing_slash(name){
 }
 
 //Provides: caml_current_dir
-//Requires: caml_trailing_slash
-if(joo_global_object.process && joo_global_object.process.cwd)
-  var caml_current_dir = joo_global_object.process.cwd().replace(/\\/g,'/');
+//Requires: caml_trailing_slash, fs_node_supported
+if(fs_node_supported () && globalThis.process && globalThis.process.cwd)
+  var caml_current_dir = globalThis.process.cwd().replace(/\\/g,'/');
 else
   var caml_current_dir =  "/static";
 caml_current_dir = caml_trailing_slash(caml_current_dir);
@@ -48,6 +48,7 @@ var caml_root = caml_get_root(caml_current_dir) || caml_failwith("unable to comp
 function MlFile(){  }
 
 //Provides: path_is_absolute
+//Requires: fs_node_supported
 function make_path_is_absolute() {
   function posix(path) {
     if (path.charAt(0) === '/') return ["", path.substring(1)];
@@ -69,8 +70,8 @@ function make_path_is_absolute() {
     }
     return;
   }
-  if(joo_global_object.process && joo_global_object.process.platform) {
-    return joo_global_object.process.platform === 'win32' ? win32 : posix;
+  if(fs_node_supported () && globalThis.process && globalThis.process.platform) {
+    return globalThis.process.platform === 'win32' ? win32 : posix;
   }
   else return posix
 }
@@ -120,7 +121,7 @@ function caml_list_mount_point(){
 }
 
 //Provides: resolve_fs_device
-//Requires: caml_make_path, jsoo_mount_point, caml_raise_sys_error, caml_get_root, MlNodeDevice, caml_trailing_slash
+//Requires: caml_make_path, jsoo_mount_point, caml_raise_sys_error, caml_get_root, MlNodeDevice, caml_trailing_slash, fs_node_supported
 function resolve_fs_device(name){
   var path = caml_make_path(name);
   var name = path.join("/");
@@ -132,7 +133,7 @@ function resolve_fs_device(name){
        && (!res || res.path.length < m.path.length))
       res = {path:m.path,device:m.device,rest:name.substring(m.path.length,name.length)};
   }
-  if( !res) {
+  if( !res && fs_node_supported()) {
     var root = caml_get_root(name);
     if (root && root.match(/^[a-zA-Z]:\/$/)){
       var m = {path:root,device:new MlNodeDevice(root)};
@@ -172,7 +173,7 @@ function caml_sys_getcwd() {
 }
 
 //Provides: caml_sys_chdir
-//Requires: caml_current_dir, caml_raise_no_such_file, resolve_fs_device, caml_trailing_slash
+//Requires: caml_current_dir, caml_raise_no_such_file, resolve_fs_device, caml_trailing_slash, caml_jsbytes_of_string
 function caml_sys_chdir(dir) {
   var root = resolve_fs_device(dir);
   if(root.device.exists(root.rest)) {
@@ -181,23 +182,19 @@ function caml_sys_chdir(dir) {
     return 0;
   }
   else {
-    caml_raise_no_such_file(dir);
+    caml_raise_no_such_file(caml_jsbytes_of_string(dir));
   }
 }
 
 //Provides: caml_raise_no_such_file
 //Requires: caml_raise_sys_error
-//Requires: caml_jsbytes_of_string
 function caml_raise_no_such_file(name){
-  name = caml_jsbytes_of_string(name);
   caml_raise_sys_error (name + ": No such file or directory");
 }
 
 //Provides: caml_raise_not_a_dir
 //Requires: caml_raise_sys_error
-//Requires: caml_jsbytes_of_string
 function caml_raise_not_a_dir(name){
-  name = caml_jsbytes_of_string(name);
   caml_raise_sys_error (name + ": Not a directory");
 }
 
@@ -222,11 +219,11 @@ function caml_sys_read_directory(name){
 }
 
 //Provides: caml_sys_remove
-//Requires: caml_raise_no_such_file, resolve_fs_device
+//Requires: caml_raise_no_such_file, resolve_fs_device, caml_jsbytes_of_string
 function caml_sys_remove(name){
   var root = resolve_fs_device(name);
   var ok = root.device.unlink(root.rest);
-  if(ok == 0) caml_raise_no_such_file(name);
+  if(ok == 0) caml_raise_no_such_file(caml_jsbytes_of_string(name));
   return 0;
 }
 
@@ -279,45 +276,53 @@ function caml_ba_map_file_bytecode(argv,argn){
   return caml_ba_map_file(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5]);
 }
 
-//Provides: caml_create_file_extern
-function caml_create_file_extern(name,content){
-  if(joo_global_object.caml_create_file)
-    joo_global_object.caml_create_file(name,content);
+//Provides: jsoo_create_file_extern
+function jsoo_create_file_extern(name,content){
+  if(globalThis.jsoo_create_file)
+    globalThis.jsoo_create_file(name,content);
   else {
-    if(!joo_global_object.caml_fs_tmp) joo_global_object.caml_fs_tmp = [];
-    joo_global_object.caml_fs_tmp.push({name:name,content:content});
+    if(!globalThis.caml_fs_tmp) globalThis.caml_fs_tmp = [];
+    globalThis.caml_fs_tmp.push({name:name,content:content});
   }
   return 0;
 }
 
 //Provides: caml_fs_init
-//Requires: caml_create_file
+//Requires: jsoo_create_file
 function caml_fs_init (){
-  var tmp=joo_global_object.caml_fs_tmp
+  var tmp=globalThis.caml_fs_tmp
   if(tmp){
     for(var i = 0; i < tmp.length; i++){
-      caml_create_file(tmp[i].name,tmp[i].content);
+      jsoo_create_file(tmp[i].name,tmp[i].content);
     }
   }
-  joo_global_object.caml_create_file = caml_create_file;
-  joo_global_object.caml_fs_tmp = [];
+  globalThis.jsoo_create_file = jsoo_create_file;
+  globalThis.caml_fs_tmp = [];
   return 0;
 }
 
 //Provides: caml_create_file
-//Requires: caml_failwith, resolve_fs_device, caml_string_of_jsbytes
+//Requires: caml_failwith, resolve_fs_device
 function caml_create_file(name,content) {
-  var name = (typeof name == "string")?caml_string_of_jsbytes(name):name;
-  var content = (typeof content == "string")?caml_string_of_jsbytes(content):content;
   var root = resolve_fs_device(name);
   if(! root.device.register) caml_failwith("cannot register file");
   root.device.register(root.rest,content);
   return 0;
 }
 
+
+//Provides: jsoo_create_file
+//Requires: caml_create_file, caml_string_of_jsbytes
+function jsoo_create_file(name,content) {
+  var name = caml_string_of_jsbytes(name);
+  var content = caml_string_of_jsbytes(content);
+  return caml_create_file(name, content);
+}
+
+
 //Provides: caml_read_file_content
 //Requires: resolve_fs_device, caml_raise_no_such_file, caml_create_bytes, caml_string_of_bytes
-//Requires: caml_string_of_jsbytes
+//Requires: caml_string_of_jsbytes, caml_jsbytes_of_string
 function caml_read_file_content (name) {
   var name = (typeof name == "string")?caml_string_of_jsbytes(name):name;
   var root = resolve_fs_device(name);
@@ -328,5 +333,5 @@ function caml_read_file_content (name) {
     file.read(0,buf,0,len);
     return caml_string_of_bytes(buf)
   }
-  caml_raise_no_such_file(name);
+  caml_raise_no_such_file(caml_jsbytes_of_string(name));
 }
