@@ -69,13 +69,15 @@ let args =
         | Some t -> `Snd t
         | None -> `Fst name)
   in
-  let builtin = if !runtime then Jsoo_runtime.runtime @ builtin else builtin in
+  let builtin =
+    if !runtime then Js_of_ocaml_compiler_runtime_files.runtime @ builtin else builtin
+  in
 
   List.iter builtin ~f:(fun t ->
       let filename = Js_of_ocaml_compiler.Builtins.File.name t in
       let runtimes = Js_of_ocaml_compiler.Linker.parse_builtin t in
-      List.iter runtimes ~f:(Js_of_ocaml_compiler.Linker.load_fragment ~filename));
-  Js_of_ocaml_compiler.Linker.load_files runtime_files;
+      Js_of_ocaml_compiler.Linker.load_fragments ~target_env:Isomorphic ~filename runtimes);
+  Js_of_ocaml_compiler.Linker.load_files ~target_env:Isomorphic runtime_files;
   let all = Jsoo_common.cmis args in
   let instr =
     List.map all ~f:(fun filename ->
